@@ -3,10 +3,21 @@ use std::os::raw::c_void;
 use std::ptr;
 use std::slice;
 
+use crate::bindgen_runtime::TypeName;
 use crate::{check_status, sys, JsUnknown, NapiValue, Ref, Result, Value, ValueType};
 
 pub struct JsArrayBuffer(pub(crate) Value);
 
+impl TypeName for JsArrayBuffer {
+  fn type_name() -> &'static str {
+    "ArrayBuffer"
+  }
+
+  fn value_type() -> ValueType {
+    ValueType::Object
+  }
+}
+
 pub struct JsArrayBufferValue {
   pub(crate) value: JsArrayBuffer,
   len: usize,
@@ -15,16 +26,36 @@ pub struct JsArrayBufferValue {
 
 pub struct JsTypedArray(pub(crate) Value);
 
+impl TypeName for JsTypedArray {
+  fn type_name() -> &'static str {
+    "TypedArray"
+  }
+
+  fn value_type() -> ValueType {
+    ValueType::Object
+  }
+}
+
 pub struct JsTypedArrayValue {
   pub arraybuffer: JsArrayBuffer,
   data: *mut c_void,
-  pub byte_offset: u64,
-  pub length: u64,
+  pub byte_offset: usize,
+  pub length: usize,
   pub typedarray_type: TypedArrayType,
 }
 
 pub struct JsDataView(pub(crate) Value);
 
+impl TypeName for JsDataView {
+  fn type_name() -> &'static str {
+    "DataView"
+  }
+
+  fn value_type() -> ValueType {
+    ValueType::Object
+  }
+}
+
 pub struct JsDataViewValue {
   pub arraybuffer: JsArrayBuffer,
   _data: *mut c_void,
@@ -57,19 +88,19 @@ pub enum TypedArrayType {
 impl From<sys::napi_typedarray_type> for TypedArrayType {
   fn from(value: sys::napi_typedarray_type) -> Self {
     match value {
-      sys::TypedarrayType::napi_int8_array => Self::Int8,
-      sys::TypedarrayType::napi_uint8_array => Self::Uint8,
-      sys::TypedarrayType::napi_uint8_clamped_array => Self::Uint8Clamped,
-      sys::TypedarrayType::napi_int16_array => Self::Int16,
-      sys::TypedarrayType::napi_uint16_array => Self::Uint16,
-      sys::TypedarrayType::napi_int32_array => Self::Int32,
-      sys::TypedarrayType::napi_uint32_array => Self::Uint32,
-      sys::TypedarrayType::napi_float32_array => Self::Float32,
-      sys::TypedarrayType::napi_float64_array => Self::Float64,
+      sys::TypedarrayType::int8_array => Self::Int8,
+      sys::TypedarrayType::uint8_array => Self::Uint8,
+      sys::TypedarrayType::uint8_clamped_array => Self::Uint8Clamped,
+      sys::TypedarrayType::int16_array => Self::Int16,
+      sys::TypedarrayType::uint16_array => Self::Uint16,
+      sys::TypedarrayType::int32_array => Self::Int32,
+      sys::TypedarrayType::uint32_array => Self::Uint32,
+      sys::TypedarrayType::float32_array => Self::Float32,
+      sys::TypedarrayType::float64_array => Self::Float64,
       #[cfg(feature = "napi6")]
-      sys::TypedarrayType::napi_bigint64_array => Self::BigInt64,
+      sys::TypedarrayType::bigint64_array => Self::BigInt64,
       #[cfg(feature = "napi6")]
-      sys::TypedarrayType::napi_biguint64_array => Self::BigUint64,
+      sys::TypedarrayType::biguint64_array => Self::BigUint64,
       _ => Self::Unknown,
     }
   }
@@ -83,13 +114,11 @@ impl From<TypedArrayType> for sys::napi_typedarray_type {
 
 impl JsArrayBuffer {
   #[cfg(feature = "napi7")]
-  #[inline]
   pub fn detach(self) -> Result<()> {
     check_status!(unsafe { sys::napi_detach_arraybuffer(self.0.env, self.0.value) })
   }
 
   #[cfg(feature = "napi7")]
-  #[inline]
   pub fn is_detached(&self) -> Result<bool> {
     let mut is_detached = false;
     check_status!(unsafe {
@@ -98,7 +127,6 @@ impl JsArrayBuffer {
     Ok(is_detached)
   }
 
-  #[inline]
   pub fn into_value(self) -> Result<JsArrayBufferValue> {
     let mut data = ptr::null_mut();
     let mut len: usize = 0;
@@ -112,7 +140,6 @@ impl JsArrayBuffer {
     })
   }
 
-  #[inline]
   pub fn into_typedarray(
     self,
     typedarray_type: TypedArrayType,
@@ -137,7 +164,6 @@ impl JsArrayBuffer {
     }))
   }
 
-  #[inline]
   pub fn into_dataview(self, length: usize, byte_offset: usize) -> Result<JsDataView> {
     let mut dataview_value = ptr::null_mut();
     check_status!(unsafe {
@@ -156,24 +182,20 @@ impl JsArrayBuffer {
     }))
   }
 
-  #[inline]
   pub fn into_ref(self) -> Result<Ref<JsArrayBufferValue>> {
     Ref::new(self.0, 1, self.into_value()?)
   }
 }
 
 impl JsArrayBufferValue {
-  #[inline]
   pub fn new(value: JsArrayBuffer, data: *mut c_void, len: usize) -> Self {
     JsArrayBufferValue { value, len, data }
   }
 
-  #[inline]
   pub fn into_raw(self) -> JsArrayBuffer {
     self.value
   }
 
-  #[inline]
   pub fn into_unknown(self) -> JsUnknown {
     unsafe { JsUnknown::from_raw_unchecked(self.value.0.env, self.value.0.value) }
   }
@@ -207,25 +229,24 @@ impl DerefMut for JsArrayBufferValue {
 
 impl JsTypedArray {
   /// get TypeArray info
-  /// https://nodejs.org/api/n-api.html#n_api_napi_get_typedarray_info
+  /// <https://nodejs.org/api/n-api.html#n_api_napi_get_typedarray_info>
   ///
   /// ***Warning***: Use caution while using this API since the underlying data buffer is managed by the VM.
-  #[inline]
   pub fn into_value(self) -> Result<JsTypedArrayValue> {
     let mut typedarray_type = 0;
-    let mut len = 0u64;
+    let mut len = 0;
     let mut data = ptr::null_mut();
     let mut arraybuffer_value = ptr::null_mut();
-    let mut byte_offset = 0u64;
+    let mut byte_offset = 0;
     check_status!(unsafe {
       sys::napi_get_typedarray_info(
         self.0.env,
         self.0.value,
         &mut typedarray_type,
-        &mut len as *mut u64 as *mut _,
+        &mut len,
         &mut data,
         &mut arraybuffer_value,
-        &mut byte_offset as *mut u64 as *mut usize,
+        &mut byte_offset,
       )
     })?;
 
@@ -239,37 +260,53 @@ impl JsTypedArray {
   }
 }
 
+impl JsTypedArrayValue {
+  #[inline]
+  fn is_valid_as_ref(&self, dest_type: TypedArrayType) {
+    if self.typedarray_type == TypedArrayType::Uint8Clamped && dest_type != TypedArrayType::Uint8 {
+      return;
+    }
+    if self.typedarray_type != dest_type {
+      panic!(
+        "invalid typedarray type: expected {:?}, got {:?}",
+        dest_type, self.typedarray_type
+      );
+    }
+  }
+}
+
 macro_rules! impl_as_ref {
-  ($ref_type:ident) => {
+  ($ref_type:ident, $expect_type:expr) => {
     impl AsRef<[$ref_type]> for JsTypedArrayValue {
       fn as_ref(&self) -> &[$ref_type] {
-        unsafe { slice::from_raw_parts(self.data as *const $ref_type, self.length as usize) }
+        self.is_valid_as_ref($expect_type);
+        unsafe { slice::from_raw_parts(self.data as *const $ref_type, self.length) }
       }
     }
 
     impl AsMut<[$ref_type]> for JsTypedArrayValue {
       fn as_mut(&mut self) -> &mut [$ref_type] {
-        unsafe { slice::from_raw_parts_mut(self.data as *mut $ref_type, self.length as usize) }
+        self.is_valid_as_ref($expect_type);
+        unsafe { slice::from_raw_parts_mut(self.data as *mut $ref_type, self.length) }
       }
     }
   };
 }
 
-impl_as_ref!(u8);
-impl_as_ref!(i8);
-impl_as_ref!(u16);
-impl_as_ref!(i16);
-impl_as_ref!(u32);
-impl_as_ref!(i32);
-impl_as_ref!(f32);
-impl_as_ref!(f64);
+impl_as_ref!(u8, TypedArrayType::Uint8);
+impl_as_ref!(i8, TypedArrayType::Int8);
+impl_as_ref!(u16, TypedArrayType::Uint16);
+impl_as_ref!(i16, TypedArrayType::Int16);
+impl_as_ref!(u32, TypedArrayType::Uint32);
+impl_as_ref!(i32, TypedArrayType::Int32);
+impl_as_ref!(f32, TypedArrayType::Float32);
+impl_as_ref!(f64, TypedArrayType::Float64);
 #[cfg(feature = "napi6")]
-impl_as_ref!(i64);
+impl_as_ref!(i64, TypedArrayType::BigInt64);
 #[cfg(feature = "napi6")]
-impl_as_ref!(u64);
+impl_as_ref!(u64, TypedArrayType::BigUint64);
 
 impl JsDataView {
-  #[inline]
   pub fn into_value(self) -> Result<JsDataViewValue> {
     let mut length = 0u64;
     let mut byte_offset = 0u64;
